﻿; *****************************************************************************
; MemDll.pbi
; by Luis
; http://luis.no-ip.net
;
; Load a DLL from memory.
; OS: Windows
; Build: x86, x64
;
; 1.00                                                 April 1, 2016    PB 5.42
; First release
;
; 1.01                                                 June 30, 2017    PB 5.60
; Converted to Module.
; Fixed a bug in CopySections().
; *****************************************************************************

; With this Module is possible to load a DLL from memory instead of from file.
; Tested on Windows 7 x64 with just *ONE* DLL both in 32 and 64 bit flavours, so probably more testing is required.
; Feel free to report if a DLL didn't work for you and I'll try to improve the code if I can.
; Worked apparently well for the limited use I made of it.

; Thanks to:

; Joachim Bauch (http://www.joachim-bauch.de) for the original code I studied to learn something about what a loader should do.
; The user "shebaw" (http://www.rohitab.com forum) for some nice C snippets I converted to PB.
; Countless Microsoft articles and text files describing the PE format.
; The x64dbg debugger (http://x64dbg.com) used to track a problem

; What this loader does:

; 1) Check if the image in memory looks like a valid DLL.
; 2) Reserve enough virtual address space to map the DLL to it, at the preferred address specified in the DLL if possible, else at a different one.
; 3) Map the DLL to virtual space, copies the headers and the sections in memory to the appropriate aligned starting offsets.
; 4) Process the import table, load the required DLLs (dependencies) and patch the table with the loaded function addresses.
; 5) If any of functions is forwarded to another DLL, the target DLL module is retrieved if the DLL is already in memory, else the required DLL is loaded.
; 6) If the DLL has not been mapped to the preferred location, the relocation table is processed and relocations are applied.
; 7) The protection flags specified in the section headers are applied to the sections in memory.
; 8) The #DLL_PROCESS_ATTACH event is signaled to the DLL and its handle is returned.

; Limitations:

; It's not possible to load from memory a DLL "A" depending on a DLL "B" if "B" is also embedded in memory and not available in the filesystem.
; In other words you can:
; - Load all the DLLs you want from memory as long they are not dependent on each other.
; - Load all the DLLs you want from memory as long their dependencies are available in the file system (tipically OS DLLs : GDI32, KERNEL32, MSVCRT, ...)

; The idea is to embed just 1 or 2 DLLs in your program since sometimes may be annoying to have to distribute them separately (for example for a small techdemo, intro, utility etc.)
; If your program requires many DLLs probably is better to just distribute them as external files along with your program.

; Usage example:

; IncludeFile "MemDll.pbi"
;
; DataSection
;  glfw3:
;  IncludeBinary "glfw3.x86.dll"
; EndDataSection
;
; hglfw = MemDLL::LoadLibrary(?glfw3)
; 
; If hglfw
;     PrototypeC.i glfwInit() : Global glfwInit.glfwInit = MemDLL::GetProcAddress(hglfw, "glfwInit")
;     Debug glfwInit()
;     MemDLL::FreeLibrary(hglfw)
; EndIf


DeclareModule MemDll 
Declare.i 	LoadLibrary (*image)
Declare.i 	GetProcAddress (hdll, func$)
Declare 	FreeLibrary (hdll)
EndDeclareModule

IncludeFile "MemDll.pb"
; IDE Options = PureBasic 5.62 (Windows - x86)
; CursorPosition = 74
; Folding = -
; EnableXP
; EnableUser
; CPU = 1
; CompileSourceDirectory
; EnablePurifier
; EnableExeConstant